package require counted

package require Itcl
namespace import itcl::*

class Account {
    public variable balance 0

    destructor {
        puts "Account::destructor"
    }

    public method deposit {amount} {
        set balance [expr $balance + $amount]
    }

    public method withdraw {amount} {
        set balance [expr $balance - $amount]
    }
}

proc createAccount {} {
    set account [Account #auto]

    # Create a reference counted object that delegates operations to an Itcl
    # object.  When the last reference is released, destroy the Itcl object.
    return [::counted::command $account [list delete object $account]]
}

set account1 [createAccount]

# The internal representation type is "cmdName".
puts "Internal representation type is [::counted::type $account1]"
puts "balance is [$account1 cget -balance]"
$account1 deposit 30
puts "balance is [$account1 cget -balance]"

# This command changes the internal representation type to "string".
puts "$account1 length is [string length $account1]"
puts "Internal representation type is [::counted::type $account1]"

# This command restores the internal representation type to "cmdName".
$account1 withdraw 20
puts "Internal representation type is [::counted::type $account1]"
puts "balance is [$account1 cget -balance]"

# Add another reference.
set account2 $account1
puts "balance is [$account2 cget -balance]"

# Release original reference.
unset account1
puts "balance is [$account2 cget -balance]"

# Release the last reference.
unset account2
