// $Id: HandleSupport.h,v 1.24 2002/03/04 23:59:39 cthuang Exp $
#ifndef CMDNAMETYPE_H
#define CMDNAMETYPE_H

#include <tcl.h>
#include <string>
#include "mutex.h"
#include "Singleton.h"

// This is the object that a handle maps to.

class InternalRep
{
protected:
    Tcl_Interp *m_interp;

    // token for created command
    Tcl_Command m_command;

    // entry in handle name to object map
    Tcl_HashEntry *m_pNameEntry;

    // number of Tcl_Obj instances that are handles to this object
    long m_handleCount;

public:
    InternalRep(Tcl_Interp *interp, Tcl_ObjCmdProc *pCmdProc);
    virtual ~InternalRep();

    // Get handle string representation for the object.
    std::string name() const;

    void incrHandleCount();
    long decrHandleCount();
};

// Handles are instances of Tcl's cmdName type which this class hijacks in
// order to map handles to application objects.

class CmdNameType
{
    // pointer to Tcl cmdName type
    static Tcl_ObjType *ms_pCmdNameType;

    // saved Tcl cmdName type
    static Tcl_ObjType ms_oldCmdNameType;

    // Tcl type functions
    static void freeInternalRep(Tcl_Obj *pObj);
    static void dupInternalRep(Tcl_Obj *pSrc, Tcl_Obj *pDup);
    static void updateString(Tcl_Obj *pObj);
    static int setFromAny(Tcl_Interp *interp, Tcl_Obj *pObj);

    friend class Singleton<CmdNameType>;
    static Singleton<CmdNameType> ms_singleton;

    CmdNameType();
    ~CmdNameType();

public:
    // Get instance of this class.
    static CmdNameType &instance();

    // Create handle.
    Tcl_Obj *newObj(Tcl_Interp *interp, InternalRep *pRep);
};

// Maps handle name to internal representation.  There's an instance of this
// class associated with each Tcl interpreter that loads the extension.

class HandleNameToRepMap
{
    // handle string representation to internal representation map
    Tcl_HashTable m_handleToRepMap;

    static void deleteInterpProc(ClientData clientData, Tcl_Interp *interp);
    static void exitProc(ClientData clientData);

    ~HandleNameToRepMap();

public:
    HandleNameToRepMap(Tcl_Interp *interp);

    // Get instance associated with the Tcl interpreter.
    static HandleNameToRepMap *instance(Tcl_Interp *interp);

    // Insert handle to object mapping.
    Tcl_HashEntry *insert(const char *handleStr, InternalRep *pRep);

    // Get the object represented by the handle.
    InternalRep *find(Tcl_Obj *pHandle) const;

    // Remove handle to object mapping.
    void erase(Tcl_HashEntry *pNameEntry);

    // Delete all handles.
    void clear();
};

#endif
