// $Id: Command.cpp,v 1.64 2002/03/06 00:56:39 cthuang Exp $
#pragma warning(disable: 4786)
#include "Command.h"

static int
commandObjCmd (
    ClientData clientData,
    Tcl_Interp *interp,
    int objc,
    Tcl_Obj *CONST objv[])
{
    Command *pCommand = static_cast<Command *>(clientData);
    return pCommand->execute(objc - 1, objv + 1);
}

Command::Command (
    Tcl_Interp *interp, Tcl_Obj *pServant, Tcl_Obj *pDestructor):
        InternalRep(interp, commandObjCmd),
        m_pServant(pServant),
        m_pDestructor(pDestructor)
{
    Tcl_IncrRefCount(m_pServant);

    if (m_pDestructor != 0) {
        Tcl_IncrRefCount(m_pDestructor);
    }
}

Command::~Command()
{
    Tcl_DecrRefCount(m_pServant);

    if (m_pDestructor != 0) {
        if (!Tcl_InterpDeleted(m_interp)) {
            Tcl_EvalObjEx(
                m_interp, m_pDestructor, TCL_EVAL_DIRECT | TCL_EVAL_GLOBAL);
        }
        Tcl_DecrRefCount(m_pDestructor);
    }
}

int
Command::execute (int objc, Tcl_Obj *CONST objv[])
{
    int completionCode;

    // Construct command by appending arguments to servant.
    Tcl_Obj *pScript = Tcl_NewListObj(1, &m_pServant);
    Tcl_IncrRefCount(pScript);

    int length;
    completionCode = Tcl_ListObjLength(m_interp, pScript, &length);
    if (completionCode == TCL_OK) {
	completionCode = Tcl_ListObjReplace(
            m_interp, pScript, length, 0, objc, objv);
    }

    if (completionCode == TCL_OK) {
        completionCode = Tcl_EvalObjEx(
            m_interp, pScript, TCL_EVAL_DIRECT | TCL_EVAL_GLOBAL);
    }

    Tcl_DecrRefCount(pScript);
    return completionCode;
}
