// $Id: main.cpp,v 1.63 2002/02/27 01:58:45 cthuang Exp $
#pragma warning(disable: 4786)
#include "CmdNameType.h"
#include "Command.h"

#define NAMESPACE "::counted::"
#define PACKAGE_NAME "counted"
#define PACKAGE_VERSION "1.0"

// This Tcl command deletes all command objects.

static int
clearCmd (
    ClientData,
    Tcl_Interp *interp,
    int /*objc*/,
    Tcl_Obj *CONST /*objv*/[])
{
    HandleNameToRepMap::instance(interp)->clear();
    return TCL_OK;
}

// This Tcl command constructs a reference counted object.

static int
commandCmd (
    ClientData,
    Tcl_Interp *interp,
    int objc,
    Tcl_Obj *CONST objv[])
{
    if (objc < 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "servant ?destructor?");
	return TCL_ERROR;
    }

    Tcl_Obj *pServant = objv[1];
    Tcl_Obj *pDestructor = (objc == 3) ? objv[2] : 0;

    Command *pCommand = new Command(interp, pServant, pDestructor);

    Tcl_SetObjResult(
        interp,
        CmdNameType::instance().newObj(interp, pCommand));
    return TCL_OK;
}

// This Tcl command returns the argument's internal representation type.

static int
typeCmd (
    ClientData,
    Tcl_Interp *interp,
    int objc,
    Tcl_Obj *CONST objv[])
{
    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "value");
	return TCL_ERROR;
    }

    Tcl_ObjType *pType = objv[1]->typePtr;
    char *name = (pType == 0) ? "NULL" : pType->name;
    Tcl_SetResult(interp, name, TCL_STATIC);
    return TCL_OK;
}

extern "C" DLLEXPORT int
Counted_Init (Tcl_Interp *interp)
{
#if TCL_MINOR_VERSION >= 1
    // Stubs were introduced in Tcl 8.1.
    if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
        return TCL_ERROR;
    }
#endif

    // Initialize handle support.
    CmdNameType::instance();
    new HandleNameToRepMap(interp);

    // Create additional commands.
    Tcl_CreateObjCommand(interp, NAMESPACE "clear", clearCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "command", commandCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "type", typeCmd, 0, 0);

    return Tcl_PkgProvide(interp, PACKAGE_NAME, PACKAGE_VERSION);
}

extern "C" DLLEXPORT int
Counted_SafeInit (Tcl_Interp *interp)
{
    return Counted_Init(interp);
}
