// $Id: mutex.h,v 1.4 1999/07/20 17:42:19 chuang Exp $
#ifndef MUTEX_H
#define MUTEX_H

#include <tcl.h>

#ifdef TCL_THREADS

// This class ensures a mutex is initialized before using it.

class Mutex
{
    Tcl_Mutex m_mutex;

public:
    Mutex ():
        m_mutex(0)
    { }

    operator Tcl_Mutex * () const
    { return const_cast<Tcl_Mutex *>(&m_mutex); }
};

// This class locks a mutex when constructed and unlocks it when destroyed.

class SingleLock
{
    Tcl_Mutex *m_pMutex;

public:
    SingleLock (Tcl_Mutex *pMutex):
        m_pMutex(pMutex)
    { Tcl_MutexLock(m_pMutex); }

    ~SingleLock ()
    { Tcl_MutexUnlock(m_pMutex); }
};

#define DECLARE_MUTEX(name) Mutex name;
#define LOCK_MUTEX(pMutex) SingleLock criticalSectionLock(pMutex);

#else

#define DECLARE_MUTEX(name)
#define LOCK_MUTEX(pMutex)

#endif

#endif
